# include <stdio.h>
# include <math.h>
# include "mpi.h"

/* Evaluation of the inregration function
   f(x)=1/(1+x*x) (0<=x<=1) */
double f(double x)
{
   double value;
   value=4.0/(1.0+x*x);
   return value;
}

int main (int argc, char *argv[])
{
   int counter;
   double walltime_start,walltime_end,walltime_diff;

   int master_node = 0;
   int number_of_nodes;
   int node_id;

   int number_of_intervals=300000000;
   double node_integral;
   double step;

   double x;

   double pi_estimate;
   double pi_diff;
   double pi_exact = 3.141592653589793238462643;

/* Establish the MPI environment */
   MPI_Init (&argc, &argv);
/* Get the number of processes */
   MPI_Comm_size (MPI_COMM_WORLD,&number_of_nodes);
/* Determine this processes's rank */
   MPI_Comm_rank (MPI_COMM_WORLD,&node_id);

   if (node_id==master_node)
   {
      printf("Compiled on %s at %s\n",__DATE__, __TIME__);
      printf("Number of nodes: %d\n",number_of_nodes);
      printf ( "Number of intervals: %d\n",number_of_intervals);
      walltime_start=MPI_Wtime();
   }

   MPI_Bcast (&number_of_nodes,1,MPI_INT,master_node,MPI_COMM_WORLD );

   step=1.0/(double)number_of_intervals;
   node_integral=0.0;
   for (counter=node_id+1;counter<=number_of_intervals;counter=counter+number_of_nodes)
   {
      x=step*((double)counter-0.5);
      node_integral=node_integral+f(x);
   }

   printf("\nNode: %d\n",node_id);
   printf("Node integral: %f\n",node_integral*step);

   MPI_Reduce (&node_integral,&pi_estimate,1,MPI_DOUBLE,MPI_SUM,master_node,MPI_COMM_WORLD);

/* The master node prints the answer */
   if (node_id==master_node)
   {
      pi_estimate=pi_estimate*step;
      pi_diff=fabs(pi_estimate-pi_exact);
      printf("PI exact   : %24.16f\n", pi_exact);
      printf("PI estimate: %24.16f\n", pi_estimate);
      printf("PI diff    : %24.16f\n", pi_diff);

      walltime_end=MPI_Wtime();
      walltime_diff=walltime_end-walltime_start;
      printf ("Wall clock: %f\n", walltime_diff );
   }

/* Shut down MPI */
   MPI_Finalize();
}

